package models

import (
	"fmt"
	"strings"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

type ChartPermission struct {
	ChartPermissionId     int       `orm:"pk" description:"权限ID"`
	ChartPermissionName   string    `description:"名称"`
	PermissionName        string    `description:"权限名"`
	Sort                  int       `description:"排序"`
	Enabled               int       `description:"是否可用"`
	CreatedTime           time.Time `description:"创建时间"`
	LastUpdatedTime       time.Time `description:"更新时间"`
	TeleconferenceSort    int       `description:"电话会类型排序"`
	Remark                string    `description:"备注"`
	ClassifyName          string    `description:"分类名称"`
	ProductName           string    `description:"产品名称" `
	ProductId             int       `description:"产品ID"`
	ImageURL              string    `description:"图片地址"`
	ShowType              int       `description:"1:查研观向小程序展示"`
	IsOther               int       `description:"是否是其他，用于查研观向小程序后台展示"`
	IsReport              int       `description:"是否是报告，用于查研观向小程序前台报告展示"`
	CygxAuth              int       `description:"是否是权限，用于查研观向小程序前台权限校验"`
	PermissionType        int       `description:"1主观，2客观"`
	YbImgUrl              string    `description:"研报小程序报告列表icon"`
	ProductPermissionName string    `description:"种类权限名称"`
	PriceDrivenState      int       `description:"品种价格驱动开启状态 0-关闭 1-开启"`
	ImageUrlM             string    `description:"图片地址(查研观向移动端)"`
	ParentId              int       `description:"父级权限id"`
	IsPublic              int       `description:"是否是公有权限1:公有权限，0私有权限"`
}

type ChartPermissionView struct {
	ChartPermissionId int                    `orm:"pk" description:"权限ID"`
	PermissionName    string                 `description:"权限名"`
	Child             []*ChartPermissionView `description:"子权限"`
	IsPublic          int                    `description:"是否是公有权限1:公有权限，0私有权限"`
	Sort              int                    `description:"排序"`
}

type BySortChartPermissionView []*ChartPermissionView

func (a BySortChartPermissionView) Len() int           { return len(a) }
func (a BySortChartPermissionView) Swap(i, j int)      { a[i], a[j] = a[j], a[i] }
func (a BySortChartPermissionView) Less(i, j int) bool { return a[i].Sort < a[j].Sort }

// GetChartPermissionList 获取全部权限品种权限列表
func GetChartPermissionList() (items []*ChartPermission, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM chart_permission WHERE enabled=1 ORDER BY sort ASC`
	_, err = o.Raw(sql).QueryRows(&items)
	return
}

// GetChildChartPermissionListById 获取子品种权限列表
func GetChildChartPermissionListById(id int) (items []*ChartPermission, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM chart_permission WHERE enabled=1 AND parent_id=? ORDER BY sort ASC`
	_, err = o.Raw(sql, id).QueryRows(&items)
	return
}

// GetChartPermissionListByIds 根据品种ids获取品种权限列表
func GetChartPermissionIdsByIds(chartPermissionIds []string) (items []*ChartPermission, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM chart_permission WHERE enabled=1 `
	if len(chartPermissionIds) > 0 {
		sql += fmt.Sprintf(" AND chart_permission_id IN (%s) ", strings.Join(chartPermissionIds, ","))
	}
	sql += ` ORDER BY sort ASC `

	_, err = o.Raw(sql).QueryRows(&items)
	return
}

// GetChildChartPermissionListById 获取品种权限id列表
func GetChildChartPermissionIdsById(chartPermissionId int) (items []string, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT chart_permission_id FROM chart_permission WHERE enabled=1 AND parent_id=? ORDER BY sort ASC`
	_, err = o.Raw(sql, chartPermissionId).QueryRows(&items)
	return
}

// GetDefaultChartPermissionCount 获取默认的权限数目
func GetDefaultChartPermissionCount(productId int) (count int, err error) {
	sql := ` select count(1) COUNT from chart_permission where product_id=?`
	o := orm.NewOrm()
	err = o.Raw(sql, productId).QueryRow(&count)
	return
}
